/* ---------------------------------------------------------------
   Version 1.03, last update on 2003-Apr-4
   This program is intended as a replacement for avpatch.exe from
   the Ultima 4 VGA Upgrade:
   http://www.moongates.com/u4
   (the original avpatch.exe can't be run under DOS)

   See compile.txt for instructions on how to compile this
   program.

   The original avpatch.exe was written by Ryan Wiener.
   --------------------------------------------------------------- */

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif

#ifndef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#ifndef SEEK_SET
#define SEEK_SET 0
#endif

#ifndef SEEK_END
#define SEEK_END 2
#endif
   
#define buffer_size 10000
#define infile_size 98208
#define patch_count 7

#include <stdio.h>
#include <stdlib.h>

long get_filesize(FILE *f);

int main() {
   const char *infile_name = "avatar.exe";
   const char *outfile_name = "avatar.old";
   const long patch_offsets[patch_count] =
      {0x0fb85, 0x17564, 0x1756b, 0x17572, 0x17579, 0x17580, 0x17587};
   const unsigned char patch_bytes[patch_count] =
      {0x30, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38};

   FILE *infile, *outfile;
   unsigned char *buffer;
   long file_pos;
   int i;

   /* ---------------------------------------------------
      open avatar.exe, create avatar.old, allocate buffer
      --------------------------------------------------- */
   infile = fopen(infile_name,"rb");
   if (infile == NULL) {
      printf("Couldn't open %s\n",infile_name);
      exit(EXIT_FAILURE);
   }
   if (get_filesize(infile) != infile_size) {
      printf("%s doesn't have the right size\n",infile_name);
      fclose(infile);
      exit(EXIT_FAILURE);
   }
   outfile = fopen(outfile_name,"wb");
   if (outfile == NULL) {
      printf("Couldn't open %s\n",outfile_name);
      fclose(infile);
      exit(EXIT_FAILURE);
   }

   buffer = (unsigned char *) malloc(buffer_size);
   if (buffer == NULL) {
      printf("Couldn't allocate buffer\n");
      fclose(infile);
      fclose(outfile);
      remove(outfile_name); /* delete the 0-byte outfile */
      exit(EXIT_FAILURE);
   }

   /* ----------------
      patch avatar.exe
      ---------------- */
   file_pos = 0;

   /* patch buffer-sized blocks */
   while (infile_size - file_pos >= buffer_size) {
      fread(buffer, 1, buffer_size, infile);
      for (i = 0; i < patch_count; i++) {
         if ((file_pos <= patch_offsets[i]) && (patch_offsets[i] < file_pos + buffer_size))
            buffer[patch_offsets[i]-file_pos] = patch_bytes[i];
      }
      fwrite(buffer, 1, buffer_size, outfile);
      file_pos += buffer_size;
   }

   /* patch the rest of the file, which is smaller than the buffer */
   if (infile_size - file_pos > 0) {
      fread(buffer, 1, (infile_size - file_pos), infile);
      for (i = 0; i < patch_count; i++) {
         if ((file_pos <= patch_offsets[i]) && (patch_offsets[i] < file_pos + buffer_size))
            buffer[patch_offsets[i]-file_pos] = patch_bytes[i];
      }
      fwrite(buffer, 1, (infile_size - file_pos), outfile);
   }

   /* --------------------------------------------------
      deallocate buffer, close avatar.exe and avatar.old
      -------------------------------------------------- */
   free(buffer);
   fclose(infile);
   fclose(outfile);

   return(EXIT_SUCCESS);
}

long get_filesize(FILE *f) {
   long filepos, filesize;

   filepos = ftell(f);
   fseek(f, 0, SEEK_END);
   filesize = ftell(f);
   fseek(f, filepos, SEEK_SET);

   return(filesize);
}
